package com.starcharge.wios.controller;

import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.convert.DeliveryConvert;
import com.starcharge.wios.dao.entity.Delivery;
import com.starcharge.wios.dao.entity.WallboxApply;
import com.starcharge.wios.dao.mappers.DeliveryMapper;
import com.starcharge.wios.dao.mappers.TauxiliarymaterialsMapper;
import com.starcharge.wios.dao.mappers.WallboxApplyMapper;
import com.starcharge.wios.dto.DeliveryUpdateDTO;
import com.starcharge.wios.enums2.ProductType;
import com.starcharge.wios.enums2.TauxiliarymaterialStatusEnum;
import com.starcharge.wios.service.CommonUpdateService;
import com.starcharge.wios.service.SnowFlakeService;
import com.starcharge.wios.utils.ExcelUtils;
import com.starcharge.wios.utils.NumberUtil;
import com.starcharge.wios.validation.CreateEntityGroup;
import com.starcharge.wios.validation.ParamsValidate;
import com.starcharge.wios.validation.UpdateEntityGroup;
import com.starcharge.wios.vo.DeliveryAddVo;
import com.starcharge.wios.vo.DeliveryExcelVo;
import io.netty.util.internal.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * @projectName:xr-wios
 * @packageName:com.starcharge.wios.controller
 * @Description:TODO
 * @Author:luyincheng
 * @CreateDate: 2023/10/818:10
 */
@RestController
@RequestMapping("api/delivery")
@Api(tags = "发货管理接口")
@Slf4j
public class DeliveryController {
    @Autowired
    private TokenService tokenService;
    @Autowired
    private DeliveryMapper deliveryMapper;
    @Autowired
    private CommonUpdateService commonUpdateService;
    @Autowired
    private TauxiliarymaterialsMapper tauxiliarymaterialsMapper;
    @Autowired
    private WallboxApplyMapper wallboxApplyMapper;
    @PostMapping("/add")
    @ApiOperation(value = "新增发货",notes = "新增发货，返回成功条数")
    @ParamsValidate
    @Transactional
    public BaseResponse<Integer> AddDelivery(@RequestBody @Validated(CreateEntityGroup.class) List<DeliveryAddVo> deliveryAddVoList, BindingResult bindingResult){
        //String orderid= snowFlakeService.getNextId(0,0)+"";
        return new BaseResponse(deliveryAddVoList.stream()
                .map(da->{
                    Delivery delivery= DeliveryConvert.INSTANCE.convertAddVo(da);
                    delivery.setDeliveryOrder("FH"+NumberUtil.getRandomOrderId());
                    if(!StringUtils.isEmpty(delivery.getLogisticOrder())){
                        delivery.setLogisticStatus(TauxiliarymaterialStatusEnum.已发货.name());
                    }
                    if(!ProductType.辅材.name().equals(delivery.getIsproduct())){
                        int result=this.wallboxApplyMapper.updateWallBoxTrackingStatusByOrderId("1",delivery.getInstallOrder());
                        if(result!=1){
                            throw new ServiceException("400","更新订单发货状态失败！订单号："+delivery.getInstallOrder());
                        }
                    }else{
                        int result=this.tauxiliarymaterialsMapper.updateTauxiliarymaterialsLogisticStatusByApplyOrder(TauxiliarymaterialStatusEnum.已发货.name(),delivery.getInstallOrder());
                        if(result!=1){
                            throw new ServiceException("400","更新辅材订单发货状态失败！订单号："+delivery.getInstallOrder());
                        }
                    }
                    this.commonUpdateService.UpdateColumns(delivery,true);
                    return this.deliveryMapper.insertSelective(delivery);
                }).reduce(0,Integer::sum));
    }
    @PostMapping("/query")
    @ApiOperation(value = "发货列表查询")
    public BaseResponse<List<Delivery>> queryDelivery(Delivery Delivery){
        return new BaseResponse(this.deliveryMapper.selectDeliveryList(Delivery));
    }
    @GetMapping("/detail")
    @ApiOperation(value = "发货详情")
    public BaseResponse<Delivery> getdetail(@RequestParam int id){
        Delivery delivery=this.deliveryMapper.selectByPrimaryKey(id);
        if (!StringUtil.isNullOrEmpty(delivery.getRejectMaterial())){
            String [] rejectMaterial=delivery.getRejectMaterial().split(",");
            List<String> list=new ArrayList<>();
            for (String rejectMaterials:rejectMaterial){
                list.add(rejectMaterials);
            }
            delivery.setRejectMaterialList(list);
        }
        return new BaseResponse(delivery);
    }
    @GetMapping("/delete")
    @ApiOperation(value = "删除发货信息")
    public BaseResponse<Integer> delete(@RequestParam int id){
        return new BaseResponse(this.deliveryMapper.deleteByPrimaryKey(id));
    }
    @PostMapping("/update")
    @ApiOperation("更新发货单信息")
    @ParamsValidate
    @Transactional
    public BaseResponse<DeliveryUpdateDTO> update(@RequestBody @Validated(UpdateEntityGroup.class) DeliveryUpdateDTO deliveryUpdateDTO, BindingResult bindingResult){
        Delivery before=this.deliveryMapper.selectByPrimaryKey(deliveryUpdateDTO.getId());
        if(StringUtils.hasLength(before.getInstallOrder())&&!before.getInstallOrder().equals(deliveryUpdateDTO.getInstallOrder())){
            if(!ProductType.辅材.name().equals(deliveryUpdateDTO)){
                this.wallboxApplyMapper.updateWallBoxTrackingStatusByOrderId("0",before.getInstallOrder());
                this.wallboxApplyMapper.updateWallBoxTrackingStatusByOrderId("1",deliveryUpdateDTO.getInstallOrder());
            }else{
                this.tauxiliarymaterialsMapper.updateTauxiliarymaterialsLogisticStatusByApplyOrder(TauxiliarymaterialStatusEnum.未发货.name(),before.getInstallOrder());
                this.tauxiliarymaterialsMapper.updateTauxiliarymaterialsLogisticStatusByApplyOrder(TauxiliarymaterialStatusEnum.已发货.name(),deliveryUpdateDTO.getInstallOrder());
            }
        }
        if(StringUtils.isEmpty(deliveryUpdateDTO.getLogisticOrder())){
            deliveryUpdateDTO.setLogisticStatus(TauxiliarymaterialStatusEnum.未发货.name());
        }else{
            deliveryUpdateDTO.setLogisticStatus(TauxiliarymaterialStatusEnum.已发货.name());
        }
        this.commonUpdateService.UpdateColumns(deliveryUpdateDTO,false);
        return new BaseResponse(this.deliveryMapper.updateByPrimaryKeySelective(deliveryUpdateDTO));
    }

    @PostMapping("/export")
    @ApiOperation(value = "导出发货单列表")
    public void export(HttpServletResponse response, @RequestBody Delivery Delivery) throws IOException {

//        ExcelUtil<WarehouseInfo> util = new ExcelUtil<WarehouseInfo>(WarehouseInfo.class);
//        util.exportExcel(response, list, "仓库信息管理数据");
        List<DeliveryExcelVo> data = DeliveryConvert.INSTANCE.convertExcelVoList(this
                .deliveryMapper.selectDeliveryList(Delivery));
        // 输出
        ExcelUtils.write(response, "发货管理列表.xls", "发货信息", DeliveryExcelVo.class, data);
    }

    @GetMapping("/supplierIdByOrderId")
    @ApiOperation(value = "根据安装单号获取服务商")
    public BaseResponse<WallboxApply> supplierIdByOrderId(@RequestParam String orderId){
        return new BaseResponse(this.wallboxApplyMapper.selectWallboxApplyByOrderId(orderId));
    }
}
